@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -12,8 +12,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/butc/butc_xbsa.c,v 1.8.14.1 2007/07/16 20:42:05 shadow Exp $");
 
 #include <sys/types.h>
 #include <afs/stds.h>
@@ -22,10 +20,12 @@ RCSID
 #include <dlfcn.h>
 #endif
 #include <errno.h>
+#include <afs/tcdata.h>
 #include "butc_xbsa.h"
 #include <afs/butx.h>
 #include <afs/bubasics.h>
 
+#include "butc_internal.h"
 #include "error_macros.h"
 
 extern int debugLevel;
@@ -34,6 +34,22 @@ extern int debugLevel;
 char resourceType[20] = "LFS FILE SYSTEM";
 #define GOODSTR(s) ((s)?(s):"<NULL>")
 
+#ifdef NEW_XBSA
+BSA_Int16 (*XBSAInit)(long *, SecurityToken *, ObjectOwner *, char **);
+BSA_Int16 (*XBSABeginTxn)(long);
+BSA_Int16 (*XBSAEndTxn)(long, Vote);
+BSA_Int16 (*XBSATerminate)(long);
+BSA_Int16 (*XBSAQueryObject)(long, QueryDescriptor *, ObjectDescriptor *);
+BSA_Int16 (*XBSAGetObject)(long, ObjectDescriptor *, DataBlock *);
+BSA_Int16 (*XBSAEndData)(long);
+BSA_Int16 (*XBSACreateObject)(long, ObjectDescriptor *, DataBlock *);
+BSA_Int16 (*XBSADeleteObject)(long, CopyType, ObjectName *, CopyId *);
+BSA_Int16 (*XBSAMarkObjectInactive)(long, ObjectName *);
+BSA_Int16 (*XBSASendData)(long, DataBlock *);
+BSA_Int16 (*XBSAGetData)(long, DataBlock *);
+void (*XBSAQueryApiVersion)(ApiVersion *);
+BSA_Int16 (*XBSAGetEnvironment)(long, ObjectOwner *, char **);
+#else
 BSA_Int16(*XBSAInit) (BSA_UInt32 *, SecurityToken *, ObjectOwner *, char **);
 BSA_Int16(*XBSABeginTxn) (BSA_UInt32);
 BSA_Int16(*XBSAEndTxn) (BSA_UInt32, Vote);
@@ -49,8 +65,9 @@ BSA_Int16(*XBSASendData) (BSA_UInt32, DataBlock *);
 BSA_Int16(*XBSAGetData) (BSA_UInt32, DataBlock *);
 BSA_Int16(*XBSAQueryApiVersion) (ApiVersion *);
 BSA_Int16(*XBSAGetEnvironment) (BSA_UInt32, ObjectOwner *, char **);
+#endif
 
-
+void
 xbsa_error(int rc, struct butx_transactionInfo *info)
 {
     switch (rc) {
@@ -113,8 +130,9 @@ xbsa_error(int rc, struct butx_transactionInfo *info)
 afs_int32
 xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 {
+#ifndef NEW_XBSA
     void *dynlib;
-    int rc;
+#endif
 
     if (debugLevel > 98) {
 	printf("\nxbsa_MountLibraray\n");
@@ -122,6 +140,7 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 
     switch (serverType) {
     case XBSA_SERVER_TYPE_ADSM:
+#ifndef NEW_XBSA
 #if defined(AFS_AIX_ENV)
 	dynlib =
 	    dlopen("/usr/lib/libXApi.a(bsashr10.o)",
@@ -136,6 +155,7 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 	dynlib = dlopen("/usr/lib/libXApi.so", RTLD_NOW | RTLD_GLOBAL);
 #else
 	dynlib = NULL;
+#endif
 #endif
 	break;
     default:
@@ -145,15 +165,18 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 	break;
     }
 
+#ifndef NEW_XBSA
     if (dynlib == NULL) {
 	ELog(0,
 	     "xbsa_MountLibrary: The dlopen call to load the XBSA shared library failed\n");
 	return (BUTX_NOLIBRARY);
     }
+#endif
 
     memset(info, 0, sizeof(struct butx_transactionInfo));
     XBSA_SET_SERVER_TYPE(info->serverType, serverType);
 
+#ifndef NEW_XBSA
 #if defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV)
     XBSAInit = (BSA_Int16(*)
 		(BSA_UInt32 *, SecurityToken *, ObjectOwner *,
@@ -204,16 +227,46 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 	     "xbsa_MountLibrary: The dlopen call to load the XBSA shared library failed\n");
 	return (BUTX_NOLIBRARY);
     }
-
     XBSAQueryApiVersion(&(info->apiVersion));
 #endif
+#else
+#if defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_LINUX26_ENV)
+    XBSAInit                    = BSAInit;
+    XBSABeginTxn                = BSABeginTxn;
+    XBSAEndTxn                  = BSAEndTxn;
+    XBSATerminate               = BSATerminate;
+    XBSAQueryObject             = BSAQueryObject;
+    XBSAGetObject               = BSAGetObject;
+    XBSAEndData                 = BSAEndData;
+    XBSACreateObject            = BSACreateObject;
+    XBSAMarkObjectInactive      = BSAMarkObjectInactive;
+    XBSADeleteObject            = BSADeleteObject;
+    XBSASendData                = BSASendData;
+    XBSAGetData                 = BSAGetData;
+    XBSAQueryApiVersion         = BSAQueryApiVersion;
+    XBSAGetEnvironment          = BSAGetEnvironment;
 
+    if (!XBSAInit || !XBSABeginTxn || !XBSAEndTxn || !XBSATerminate ||
+	!XBSAQueryObject || !XBSAGetObject || !XBSAEndData ||
+	!XBSACreateObject || !XBSADeleteObject || !XBSAMarkObjectInactive ||
+	!XBSASendData || !XBSAGetData || !XBSAQueryApiVersion ||
+	!XBSAGetEnvironment) {
+	ELog(0,"xbsa_MountLibrary: The dlopen call to load the XBSA shared library failed\n");
+	return(BUTX_NOLIBRARY);
+    }
+    XBSAQueryApiVersion(&(info->apiVersion));
+#endif
+#endif
+
+#ifdef DEBUG_BUTC
+	printf("xbsa_MountLibrary : XBSA function Pointers initialised. \n");
+#endif
     /*
      * Verify the API version
      */
     if ((info->apiVersion.version == XBSA_TS_VERSION)
 	&& (info->apiVersion.release == XBSA_TS_RELEASE)) {
-	/* XOPEN Techincal Standard Level! 
+	/* XOPEN Techincal Standard Level!
 	 * We are coded to the Preliminary Spec!  Time to go boom!
 	 */
 	ELog(0,
@@ -330,7 +383,7 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
 	}
 	strcpy(info->objOwner.bsaObjectOwner, bsaObjectOwner);
     } else {
-	info->objOwner.bsaObjectOwner[0] = NULL;
+	info->objOwner.bsaObjectOwner[0] = '\0';
     }
 
     if (appObjectOwner) {
@@ -342,7 +395,7 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
 	}
 	strcpy(info->objOwner.appObjectOwner, appObjectOwner);
     } else {
-	info->objOwner.appObjectOwner[0] = NULL;
+	info->objOwner.appObjectOwner[0] = '\0';
     }
 
     if (secToken) {
@@ -354,7 +407,7 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
 	}
 	strcpy(info->secToken, secToken);
     } else {
-	info->secToken[0] = NULL;
+	info->secToken[0] = '\0';
     }
 
     rc = (int)XBSAInit(&(info->bsaHandle), &(info->secToken),
@@ -384,7 +437,7 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
     switch (XBSA_GET_SERVER_TYPE(info->serverType)) {
     case XBSA_SERVER_TYPE_ADSM:
 	for (i = 0; i < XBSA_NUM_ENV_STRS; i++) {
-	    if (strncmp(envP[i], ADSMMaxObject, sizeof(ADSMMaxObject)) == 0) {
+	    if (strncmp(envP[i], ADSMMaxObject, strlen(ADSMMaxObject)) == 0) {
 		tempStrPtr = envP[i];
 		tempStrPtr = tempStrPtr + strlen(ADSMMaxObject);
 		info->maxObjects = strtol(tempStrPtr, NULL, 10);
@@ -553,7 +606,7 @@ xbsa_QueryObject(struct butx_transactionInfo * info, char *objectSpaceName,
 	}
 	strcpy(queryDescriptor.objName.objectSpaceName, objectSpaceName);
     } else {
-	queryDescriptor.objName.objectSpaceName[0] = NULL;
+	queryDescriptor.objName.objectSpaceName[0] = '\0';
     }
 
     if (pathName) {
@@ -565,7 +618,7 @@ xbsa_QueryObject(struct butx_transactionInfo * info, char *objectSpaceName,
 	}
 	strcpy(queryDescriptor.objName.pathName, pathName);
     } else {
-	queryDescriptor.objName.pathName[0] = NULL;
+	queryDescriptor.objName.pathName[0] = '\0';
     }
 
     queryDescriptor.owner = info->objOwner;
@@ -695,7 +748,6 @@ xbsa_WriteObjectBegin(struct butx_transactionInfo * info,
 		      char *objectInfo)
 {
     int rc;
-    int length;
     DataBlock dataBlock;
 
     if (debugLevel > 98) {
@@ -728,7 +780,7 @@ xbsa_WriteObjectBegin(struct butx_transactionInfo * info,
 	}
 	strcpy(info->curObject.objName.objectSpaceName, objectSpaceName);
     } else {
-	info->curObject.objName.objectSpaceName[0] = NULL;
+	info->curObject.objName.objectSpaceName[0] = '\0';
     }
 
     if (pathName) {
@@ -740,7 +792,7 @@ xbsa_WriteObjectBegin(struct butx_transactionInfo * info,
 	}
 	strcpy(info->curObject.objName.pathName, pathName);
     } else {
-	info->curObject.objName.pathName[0] = NULL;
+	info->curObject.objName.pathName[0] = '\0';
     }
 
     if (lGName) {
@@ -752,7 +804,7 @@ xbsa_WriteObjectBegin(struct butx_transactionInfo * info,
 	}
 	strcpy(info->curObject.lGName, lGName);
     } else {
-	info->curObject.lGName[0] = NULL;
+	info->curObject.lGName[0] = '\0';
     }
 
     if (objectDescription) {
@@ -768,7 +820,7 @@ xbsa_WriteObjectBegin(struct butx_transactionInfo * info,
 	}
 	strcpy(info->curObject.desc, objectDescription);
     } else {
-	info->curObject.desc[0] = NULL;
+	info->curObject.desc[0] = '\0';
     }
 
     if (objectInfo) {
@@ -784,7 +836,7 @@ xbsa_WriteObjectBegin(struct butx_transactionInfo * info,
 	}
 	strcpy(info->curObject.objectInfo, objectInfo);
     } else {
-	info->curObject.objectInfo[0] = NULL;
+	info->curObject.objectInfo[0] = '\0';
     }
 
     if (info->numObjects == info->maxObjects) {
@@ -862,7 +914,7 @@ xbsa_DeleteObject(struct butx_transactionInfo * info, char *objectSpaceName,
 	}
 	strcpy(objectName.objectSpaceName, objectSpaceName);
     } else {
-	objectName.objectSpaceName[0] = NULL;
+	objectName.objectSpaceName[0] = '\0';
     }
 
     if (pathName) {
@@ -873,7 +925,7 @@ xbsa_DeleteObject(struct butx_transactionInfo * info, char *objectSpaceName,
 	}
 	strcpy(objectName.pathName, pathName);
     } else {
-	objectName.pathName[0] = NULL;
+	objectName.pathName[0] = '\0';
     }
 
     rc = (int)XBSAMarkObjectInactive(info->bsaHandle, &objectName);
