@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -96,8 +96,8 @@ main(int argc, char *argv[], char **envp)
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -142,7 +142,7 @@ static void
 getpipepass(char *gpbuf, int len)
 {
     /* read a password from stdin, stop on \n or eof */
-    register int i, tc;
+    int i, tc;
     memset(gpbuf, 0, len);
     for (i = 0; i < len; i++) {
 	tc = fgetc(stdin);
@@ -202,7 +202,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
     char instance[MAXKTCNAMELEN] = "";
     char cell[MAXKTCREALMLEN] = "";
     char realm[MAXKTCREALMLEN] = "";
-    afs_int32 serverList[MAXSERVERS];
+    afs_uint32 serverList[MAXSERVERS];
     char *lcell;		/* local cellname */
     int code;
     int i;
@@ -218,7 +218,6 @@ CommandProc(struct cmd_syndesc *as, void *arock)
     struct passwd pwent;
     struct passwd *pw = &pwent;
 
-    int insist;			/* insist on good password quality */
     int lexplicit = 0;		/* servers specified explicitly */
     int local;			/* explicit cell is same a local cell */
     int foundPassword = 0;	/*Not yet, anyway */
@@ -410,7 +409,6 @@ CommandProc(struct cmd_syndesc *as, void *arock)
     give_to_child(passwd);
 
     /* Get new password if it wasn't provided. */
-    insist = 0;
     if (!foundNewPassword) {
 	if (Pipe)
 	    getpipepass(npasswd, sizeof(npasswd));
